@@ -126,18 +126,18 @@ public:
     }
 
     /** is allocation type memory? */
-    static bool ismemory(int id) {
+    static bool ismemory(const int id) {
         return ((id > 0) && ((id & 1) == 0));
     }
-    static bool ismemory(const AllocFunc* func) {
+    static bool ismemory(const AllocFunc* const func) {
         return ((func->groupId > 0) && ((func->groupId & 1) == 0));
     }
 
     /** is allocation type resource? */
-    static bool isresource(int id) {
+    static bool isresource(const int id) {
         return ((id > 0) && ((id & 1) == 1));
     }
-    static bool isresource(const AllocFunc* func) {
+    static bool isresource(const AllocFunc* const func) {
         return ((func->groupId > 0) && ((func->groupId & 1) == 1));
     }
 
@@ -201,14 +201,14 @@ public:
         bool opLessAllowed;
 
         Action getAction(const std::string& function) const {
-            std::map<std::string, Function>::const_iterator i = functions.find(function);
+            const std::map<std::string, Function>::const_iterator i = functions.find(function);
             if (i != functions.end())
                 return i->second.action;
             return NO_ACTION;
         }
 
         Yield getYield(const std::string& function) const {
-            std::map<std::string, Function>::const_iterator i = functions.find(function);
+            const std::map<std::string, Function>::const_iterator i = functions.find(function);
             if (i != functions.end())
                 return i->second.yield;
             return NO_YIELD;
