@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2015 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2018 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,20 +21,31 @@
 #define tokenH
 //---------------------------------------------------------------------------
 
+#include "config.h"
+#include "mathlib.h"
+#include "valueflow.h"
+
+#include <cstddef>
 #include <list>
+#include <ostream>
 #include <string>
 #include <vector>
-#include <ostream>
-#include "config.h"
-#include "valueflow.h"
-#include "mathlib.h"
 
-class Scope;
-class Type;
+class Enumerator;
 class Function;
-class Variable;
-class ValueType;
+class Scope;
 class Settings;
+class Type;
+class ValueType;
+class Variable;
+
+/**
+ * @brief This struct stores pointers to the front and back tokens of the list this token is in.
+ */
+struct TokensFrontBack {
+    Token *front;
+    Token *back;
+};
 
 /// @addtogroup Core
 /// @{
@@ -44,31 +55,30 @@ class Settings;
  *
  * Tokens are stored as strings. The "if", "while", etc are stored in plain text.
  * The reason the Token class is needed (instead of using the string class) is that some extra functionality is also needed for tokens:
- *  - location of the token is stored (linenr, fileIndex)
+ *  - location of the token is stored (fileIndex, linenr, column)
  *  - functions for classifying the token (isName, isNumber, isBoolean, isStandardType)
  *
  * The Token class also has other functions for management of token list, matching tokens, etc.
  */
 class CPPCHECKLIB Token {
 private:
-    Token **tokensBack;
+    TokensFrontBack* _tokensFrontBack;
 
     // Not implemented..
-    Token();
     Token(const Token &);
     Token operator=(const Token &);
 
 public:
     enum Type {
         eVariable, eType, eFunction, eKeyword, eName, // Names: Variable (varId), Type (typeId, later), Function (FuncId, later), Language keyword, Name (unknown identifier)
-        eNumber, eString, eChar, eBoolean, eLiteral, // Literals: Number, String, Character, Boolean, User defined literal (C++11)
+        eNumber, eString, eChar, eBoolean, eLiteral, eEnumerator, // Literals: Number, String, Character, Boolean, User defined literal (C++11), Enumerator
         eArithmeticalOp, eComparisonOp, eAssignmentOp, eLogicalOp, eBitOp, eIncDecOp, eExtendedOp, // Operators: Arithmetical, Comparison, Assignment, Logical, Bitwise, ++/--, Extended
         eBracket, // {, }, <, >: < and > only if link() is set. Otherwise they are comparison operators.
         eOther,
         eNone
     };
 
-    explicit Token(Token **tokensBack);
+    explicit Token(TokensFrontBack *tokensFrontBack = nullptr);
     ~Token();
 
     template<typename T>
@@ -94,6 +104,11 @@ public:
      */
     void deleteNext(unsigned long index = 1);
 
+    /**
+    * Unlink and delete the previous 'index' tokens.
+    */
+    void deletePrevious(unsigned long index = 1);
+
     /**
      * Swap the contents of this token with the next token.
      */
@@ -150,19 +165,20 @@ public:
      *
      * Possible patterns
      * - "%any%" any token
-     * - "%name%" any token which is a name, variable or type e.g. "hello" or "int"
-     * - "%type%" Anything that can be a variable type, e.g. "int", but not "delete".
-     * - "%num%" Any numeric token, e.g. "23"
+     * - "%assign%" a assignment operand
      * - "%bool%" true or false
      * - "%char%" Any token enclosed in &apos;-character.
      * - "%comp%" Any token such that isComparisonOp() returns true.
+     * - "%cop%" Any token such that isConstOp() returns true.
+     * - "%name%" any token which is a name, variable or type e.g. "hello" or "int"
+     * - "%num%" Any numeric token, e.g. "23"
+     * - "%op%" Any token such that isOp() returns true.
+     * - "%or%" A bitwise-or operator '|'
+     * - "%oror%" A logical-or operator '||'
+     * - "%type%" Anything that can be a variable type, e.g. "int", but not "delete".
      * - "%str%" Any token starting with &quot;-character (C-string).
      * - "%var%" Match with token with varId > 0
      * - "%varid%" Match with parameter varid
-     * - "%op%" Any token such that isOp() returns true.
-     * - "%cop%" Any token such that isConstOp() returns true.
-     * - "%or%" A bitwise-or operator '|'
-     * - "%oror%" A logical-or operator '||'
      * - "[abc]" Any of the characters 'a' or 'b' or 'c'
      * - "int|void|char" Any of the strings, int, void or char
      * - "int|void|char|" Any of the strings, int, void or char or empty string
@@ -170,16 +186,9 @@ public:
      * - "someRandomText" If token contains "someRandomText".
      *
      * multi-compare patterns such as "int|void|char" can contain %%or%, %%oror% and %%op%
-     * but it is not recommended to put such an %%cmd% as the first pattern.
-     *
-     * It's possible to use multi-compare patterns with all the other %%cmds%,
-     * except for %%varid%, and normal names, but the %%cmds% should be put as
-     * the first patterns in the list, then the normal names.
+     * it is recommended to put such an %%cmd% as the first pattern.
      * For example: "%var%|%num%|)" means yes to a variable, a number or ')'.
      *
-     * @todo Make it possible to use the %%cmds% and the normal names in the
-     * multicompare list without an order.
-     *
      * The patterns can be also combined to compare to multiple tokens at once
      * by separating tokens with a space, e.g.
      * ") const|void {" will return true if first token is ')' next token is either
@@ -225,7 +234,7 @@ public:
     static std::string getCharAt(const Token *tok, std::size_t index);
 
     const ValueType *valueType() const {
-        return valuetype;
+        return _valuetype;
     }
     void setValueType(ValueType *vt);
 
@@ -233,7 +242,7 @@ public:
         const Token *top = this;
         while (top && !Token::Match(top->astParent(), ",|("))
             top = top->astParent();
-        return top ? top->valuetype : nullptr;
+        return top ? top->_valuetype : nullptr;
     }
 
     Token::Type tokType() const {
@@ -241,28 +250,38 @@ public:
     }
     void tokType(Token::Type t) {
         _tokType = t;
+
+        const bool memoizedIsName = (_tokType == eName || _tokType == eType || _tokType == eVariable ||
+                                     _tokType == eFunction || _tokType == eKeyword || _tokType == eBoolean ||
+                                     _tokType == eEnumerator); // TODO: "true"/"false" aren't really a name...
+        setFlag(fIsName, memoizedIsName);
+
+        const bool memoizedIsLiteral = (_tokType == eNumber || _tokType == eString || _tokType == eChar ||
+                                        _tokType == eBoolean || _tokType == eLiteral || _tokType == eEnumerator);
+        setFlag(fIsLiteral, memoizedIsLiteral);
     }
-    void isKeyword(bool kwd) {
+    void isKeyword(const bool kwd) {
         if (kwd)
-            _tokType = eKeyword;
+            tokType(eKeyword);
         else if (_tokType == eKeyword)
-            _tokType = eName;
+            tokType(eName);
     }
     bool isKeyword() const {
         return _tokType == eKeyword;
     }
     bool isName() const {
-        return _tokType == eName || _tokType == eType || _tokType == eVariable || _tokType == eFunction || _tokType == eKeyword ||
-               _tokType == eBoolean; // TODO: "true"/"false" aren't really a name...
+        return getFlag(fIsName);
     }
     bool isUpperCaseName() const;
     bool isLiteral() const {
-        return _tokType == eNumber || _tokType == eString || _tokType == eChar ||
-               _tokType == eBoolean || _tokType == eLiteral;
+        return getFlag(fIsLiteral);
     }
     bool isNumber() const {
         return _tokType == eNumber;
     }
+    bool isEnumerator() const {
+        return _tokType == eEnumerator;
+    }
     bool isOp() const {
         return (isConstOp() ||
                 isAssignmentOp() ||
@@ -295,25 +314,25 @@ public:
     unsigned int flags() const {
         return _flags;
     }
-    void flags(unsigned int flags_) {
+    void flags(const unsigned int flags_) {
         _flags = flags_;
     }
     bool isUnsigned() const {
         return getFlag(fIsUnsigned);
     }
-    void isUnsigned(bool sign) {
+    void isUnsigned(const bool sign) {
         setFlag(fIsUnsigned, sign);
     }
     bool isSigned() const {
         return getFlag(fIsSigned);
     }
-    void isSigned(bool sign) {
+    void isSigned(const bool sign) {
         setFlag(fIsSigned, sign);
     }
     bool isPointerCompare() const {
         return getFlag(fIsPointerCompare);
     }
-    void isPointerCompare(bool b) {
+    void isPointerCompare(const bool b) {
         setFlag(fIsPointerCompare, b);
     }
     bool isLong() const {
@@ -325,13 +344,13 @@ public:
     bool isStandardType() const {
         return getFlag(fIsStandardType);
     }
-    void isStandardType(bool b) {
+    void isStandardType(const bool b) {
         setFlag(fIsStandardType, b);
     }
     bool isExpandedMacro() const {
         return getFlag(fIsExpandedMacro);
     }
-    void isExpandedMacro(bool m) {
+    void isExpandedMacro(const bool m) {
         setFlag(fIsExpandedMacro, m);
     }
     bool isCast() const {
@@ -343,13 +362,13 @@ public:
     bool isAttributeConstructor() const {
         return getFlag(fIsAttributeConstructor);
     }
-    void isAttributeConstructor(bool ac) {
+    void isAttributeConstructor(const bool ac) {
         setFlag(fIsAttributeConstructor, ac);
     }
     bool isAttributeDestructor() const {
         return getFlag(fIsAttributeDestructor);
     }
-    void isAttributeDestructor(bool value) {
+    void isAttributeDestructor(const bool value) {
         setFlag(fIsAttributeDestructor, value);
     }
     bool isAttributeUnused() const {
@@ -361,13 +380,13 @@ public:
     bool isAttributeUsed() const {
         return getFlag(fIsAttributeUsed);
     }
-    void isAttributeUsed(bool unused) {
+    void isAttributeUsed(const bool unused) {
         setFlag(fIsAttributeUsed, unused);
     }
     bool isAttributePure() const {
         return getFlag(fIsAttributePure);
     }
-    void isAttributePure(bool value) {
+    void isAttributePure(const bool value) {
         setFlag(fIsAttributePure, value);
     }
     bool isAttributeConst() const {
@@ -379,42 +398,97 @@ public:
     bool isAttributeNoreturn() const {
         return getFlag(fIsAttributeNoreturn);
     }
-    void isAttributeNoreturn(bool value) {
+    void isAttributeNoreturn(const bool value) {
         setFlag(fIsAttributeNoreturn, value);
     }
     bool isAttributeNothrow() const {
         return getFlag(fIsAttributeNothrow);
     }
-    void isAttributeNothrow(bool value) {
+    void isAttributeNothrow(const bool value) {
         setFlag(fIsAttributeNothrow, value);
     }
+    bool isAttributePacked() const {
+        return getFlag(fIsAttributePacked);
+    }
+    void isAttributePacked(const bool value) {
+        setFlag(fIsAttributePacked, value);
+    }
+    bool isAttributeNodiscard() const {
+        return getFlag(fIsAttributeNodiscard);
+    }
+    void isAttributeNodiscard(const bool value) {
+        setFlag(fIsAttributeNodiscard, value);
+    }
+    bool isControlFlowKeyword() const {
+        return getFlag(fIsControlFlowKeyword);
+    }
     bool isOperatorKeyword() const {
         return getFlag(fIsOperatorKeyword);
     }
-    void isOperatorKeyword(bool value) {
+    void isOperatorKeyword(const bool value) {
         setFlag(fIsOperatorKeyword, value);
     }
     bool isComplex() const {
         return getFlag(fIsComplex);
     }
-    void isComplex(bool value) {
+    void isComplex(const bool value) {
         setFlag(fIsComplex, value);
     }
+    bool isEnumType() const {
+        return getFlag(fIsEnumType);
+    }
+    void isEnumType(const bool value) {
+        setFlag(fIsEnumType, value);
+    }
 
-    static const Token *findsimplematch(const Token *startTok, const char pattern[]);
-    static const Token *findsimplematch(const Token *startTok, const char pattern[], const Token *end);
-    static const Token *findmatch(const Token *startTok, const char pattern[], unsigned int varId = 0);
-    static const Token *findmatch(const Token *startTok, const char pattern[], const Token *end, unsigned int varId = 0);
-    static Token *findsimplematch(Token *startTok, const char pattern[]) {
+    bool isBitfield() const {
+        return _bits > 0;
+    }
+    unsigned char bits() const {
+        return _bits;
+    }
+    void setBits(const unsigned char b) {
+        _bits = b;
+    }
+
+    /**
+     * @brief Is current token a template argument?
+     *
+     * Original code:
+     *
+     *     template<class C> struct S {
+     *         C x;
+     *     };
+     *     S<int> s;
+     *
+     * Resulting code:
+     *
+     *     struct S<int> {
+     *         int x ;  // <- "int" is a template argument
+     *     }
+     *     S<int> s;
+     */
+    bool isTemplateArg() const {
+        return getFlag(fIsTemplateArg);
+    }
+    void isTemplateArg(const bool value) {
+        setFlag(fIsTemplateArg, value);
+    }
+
+    static const Token *findsimplematch(const Token * const startTok, const char pattern[]);
+    static const Token *findsimplematch(const Token * const startTok, const char pattern[], const Token * const end);
+    static const Token *findmatch(const Token * const startTok, const char pattern[], const unsigned int varId = 0U);
+    static const Token *findmatch(const Token * const startTok, const char pattern[], const Token * const end, const unsigned int varId = 0U);
+    static Token *findsimplematch(Token * const startTok, const char pattern[]) {
         return const_cast<Token *>(findsimplematch(const_cast<const Token *>(startTok), pattern));
     }
-    static Token *findsimplematch(Token *startTok, const char pattern[], const Token *end) {
+    static Token *findsimplematch(Token * const startTok, const char pattern[], const Token * const end) {
         return const_cast<Token *>(findsimplematch(const_cast<const Token *>(startTok), pattern, end));
     }
-    static Token *findmatch(Token *startTok, const char pattern[], unsigned int varId = 0) {
+    static Token *findmatch(Token * const startTok, const char pattern[], const unsigned int varId = 0U) {
         return const_cast<Token *>(findmatch(const_cast<const Token *>(startTok), pattern, varId));
     }
-    static Token *findmatch(Token *startTok, const char pattern[], const Token *end, unsigned int varId = 0) {
+    static Token *findmatch(Token * const startTok, const char pattern[], const Token * const end, const unsigned int varId = 0U) {
         return const_cast<Token *>(findmatch(const_cast<const Token *>(startTok), pattern, end, varId));
     }
 
@@ -425,14 +499,21 @@ public:
      * string, return value is 0. If needle was not found, return
      * value is -1.
      *
-     * @param needle Current token
+     * @param tok Current token (needle)
      * @param haystack e.g. "one|two" or "|one|two"
      * @param varid optional varid of token
      * @return 1 if needle is found from the haystack
      *         0 if needle was empty string
      *        -1 if needle was not found
      */
-    static int multiCompare(const Token *needle, const char *haystack, unsigned int varid);
+    static int multiCompare(const Token *tok, const char *haystack, unsigned int varid);
+
+    unsigned int fileIndex() const {
+        return _fileIndex;
+    }
+    void fileIndex(unsigned int indexOfFile) {
+        _fileIndex = indexOfFile;
+    }
 
     unsigned int linenr() const {
         return _linenr;
@@ -441,11 +522,11 @@ public:
         _linenr = lineNumber;
     }
 
-    unsigned int fileIndex() const {
-        return _fileIndex;
+    unsigned int col() const {
+        return _col;
     }
-    void fileIndex(unsigned int indexOfFile) {
-        _fileIndex = indexOfFile;
+    void col(unsigned int c) {
+        _col = c;
     }
 
     Token *next() const {
@@ -466,12 +547,11 @@ public:
      * Insert new token after this token. This function will handle
      * relations between next and previous token also.
      * @param tokenStr String for the new token.
+     * @param originalNameStr String used for Token::originalName().
      * @param prepend Insert the new token before this token when it's not
      * the first one on the tokens list.
      */
-    void insertToken(const std::string &tokenStr, bool prepend=false);
-
-    void insertToken(const std::string &tokenStr, const std::string &originalNameStr, bool prepend=false);
+    void insertToken(const std::string &tokenStr, const std::string &originalNameStr=emptyString, bool prepend=false);
 
     Token *previous() const {
         return _previous;
@@ -483,10 +563,12 @@ public:
     }
     void varId(unsigned int id) {
         _varId = id;
-        if (id != 0)
-            _tokType = eVariable;
-        else
+        if (id != 0) {
+            tokType(eVariable);
+            isStandardType(false);
+        } else {
             update_property_info();
+        }
     }
 
     /**
@@ -536,7 +618,7 @@ public:
      * @param end Stringification ends before this token is reached. 0 to stringify until end of list.
      * @return Stringified token list as a string
      */
-    std::string stringifyList(bool varid, bool attributes, bool linenumbers, bool linebreaks, bool files, const std::vector<std::string>* fileNames = 0, const Token* end = 0) const;
+    std::string stringifyList(bool varid, bool attributes, bool linenumbers, bool linebreaks, bool files, const std::vector<std::string>* fileNames = nullptr, const Token* end = nullptr) const;
     std::string stringifyList(const Token* end, bool attributes = true) const;
     std::string stringifyList(bool varid = false) const;
 
@@ -596,16 +678,16 @@ public:
     void function(const Function *f) {
         _function = f;
         if (f)
-            _tokType = eFunction;
+            tokType(eFunction);
         else if (_tokType == eFunction)
-            _tokType = eName;
+            tokType(eName);
     }
 
     /**
      * @return a pointer to the Function associated with this token.
      */
     const Function *function() const {
-        return _tokType == eFunction ? _function : 0;
+        return _tokType == eFunction ? _function : nullptr;
     }
 
     /**
@@ -615,35 +697,48 @@ public:
     void variable(const Variable *v) {
         _variable = v;
         if (v || _varId)
-            _tokType = eVariable;
+            tokType(eVariable);
         else if (_tokType == eVariable)
-            _tokType = eName;
+            tokType(eName);
     }
 
     /**
      * @return a pointer to the variable associated with this token.
      */
     const Variable *variable() const {
-        return _tokType == eVariable ? _variable : 0;
+        return _tokType == eVariable ? _variable : nullptr;
     }
 
     /**
     * Associate this token with given type
     * @param t Type to be associated
     */
-    void type(const ::Type *t) {
-        _type = t;
-        if (t)
-            _tokType = eType;
-        else if (_tokType == eType)
-            _tokType = eName;
-    }
+    void type(const ::Type *t);
 
     /**
     * @return a pointer to the type associated with this token.
     */
     const ::Type *type() const {
-        return _tokType == eType ? _type : 0;
+        return _tokType == eType ? _type : nullptr;
+    }
+
+    /**
+    * @return a pointer to the Enumerator associated with this token.
+    */
+    const Enumerator *enumerator() const {
+        return _tokType == eEnumerator ? _enumerator : nullptr;
+    }
+
+    /**
+     * Associate this token with given enumerator
+     * @param e Enumerator to be associated
+     */
+    void enumerator(const Enumerator *e) {
+        _enumerator = e;
+        if (e)
+            tokType(eEnumerator);
+        else if (_tokType == eEnumerator)
+            tokType(eName);
     }
 
     /**
@@ -712,6 +807,10 @@ public:
         return _originalName ? *_originalName : emptyString;
     }
 
+    const std::list<ValueFlow::Value>& values() const {
+        return _values ? *_values : emptyValueList;
+    }
+
     /**
      * Sets the original name.
      */
@@ -723,39 +822,61 @@ public:
             *_originalName = name;
     }
 
-    /** Values of token */
-    std::list<ValueFlow::Value> values;
+    bool hasKnownIntValue() const {
+        return _values && _values->size() == 1U && _values->front().isKnown() && _values->front().isIntValue();
+    }
+
+    bool hasKnownValue() const {
+        return _values && _values->size() == 1U && _values->front().isKnown();
+    }
 
     const ValueFlow::Value * getValue(const MathLib::bigint val) const {
-        std::list<ValueFlow::Value>::const_iterator it;
-        for (it = values.begin(); it != values.end(); ++it) {
-            if (it->intvalue == val && !it->tokvalue)
+        if (!_values)
+            return nullptr;
+        for (std::list<ValueFlow::Value>::const_iterator it = _values->begin(); it != _values->end(); ++it) {
+            if (it->isIntValue() && it->intvalue == val)
                 return &(*it);
         }
-        return NULL;
+        return nullptr;
     }
 
     const ValueFlow::Value * getMaxValue(bool condition) const {
+        if (!_values)
+            return nullptr;
         const ValueFlow::Value *ret = nullptr;
-        std::list<ValueFlow::Value>::const_iterator it;
-        for (it = values.begin(); it != values.end(); ++it) {
-            if (it->tokvalue)
+        for (std::list<ValueFlow::Value>::const_iterator it = _values->begin(); it != _values->end(); ++it) {
+            if (!it->isIntValue())
                 continue;
             if ((!ret || it->intvalue > ret->intvalue) &&
-                ((it->condition != NULL) == condition))
+                ((it->condition != nullptr) == condition))
                 ret = &(*it);
         }
         return ret;
     }
 
+    const ValueFlow::Value * getMovedValue() const {
+        if (!_values)
+            return nullptr;
+        for (std::list<ValueFlow::Value>::const_iterator it = _values->begin(); it != _values->end(); ++it) {
+            if (it->isMovedValue() && it->moveKind != ValueFlow::Value::NonMovedVariable)
+                return &(*it);
+        }
+        return nullptr;
+    }
+
     const ValueFlow::Value * getValueLE(const MathLib::bigint val, const Settings *settings) const;
     const ValueFlow::Value * getValueGE(const MathLib::bigint val, const Settings *settings) const;
 
+    const ValueFlow::Value * getInvalidValue(const Token *ftok, unsigned int argnr, const Settings *settings) const;
+
     const Token *getValueTokenMaxStrLength() const;
     const Token *getValueTokenMinStrSize() const;
 
     const Token *getValueTokenDeadPointer() const;
 
+    /** Add token value. Return true if value is added. */
+    bool addValue(const ValueFlow::Value &value);
+
 private:
 
     void next(Token *nextToken) {
@@ -765,6 +886,9 @@ private:
         _previous = previousToken;
     }
 
+    /** used by deleteThis() to take data from token to delete */
+    void takeData(Token *fromToken);
+
     /**
      * Works almost like strcmp() except returns only true or false and
      * if str has empty space &apos; &apos; character, that character is handled
@@ -791,11 +915,13 @@ private:
         const Function *_function;
         const Variable *_variable;
         const ::Type* _type;
+        const Enumerator *_enumerator;
     };
 
     unsigned int _varId;
     unsigned int _fileIndex;
     unsigned int _linenr;
+    unsigned int _col;
 
     /**
      * A value from 0-100 that provides a rough idea about where in the token
@@ -821,8 +947,15 @@ private:
         fIsAttributeNoreturn    = (1 << 12), // __attribute__((noreturn)), __declspec(noreturn)
         fIsAttributeNothrow     = (1 << 13), // __attribute__((nothrow)), __declspec(nothrow)
         fIsAttributeUsed        = (1 << 14), // __attribute__((used))
-        fIsOperatorKeyword      = (1 << 15), // operator=, etc
-        fIsComplex              = (1 << 16)  // complex/_Complex type
+        fIsAttributePacked      = (1 << 15), // __attribute__((packed))
+        fIsControlFlowKeyword   = (1 << 16), // if/switch/while/...
+        fIsOperatorKeyword      = (1 << 17), // operator=, etc
+        fIsComplex              = (1 << 18), // complex/_Complex type
+        fIsEnumType             = (1 << 19), // enumeration type
+        fIsName                 = (1 << 20),
+        fIsLiteral              = (1 << 21),
+        fIsTemplateArg          = (1 << 22),
+        fIsAttributeNodiscard   = (1 << 23), // __attribute__ ((warn_unused_result)), [[nodiscard]]
     };
 
     unsigned int _flags;
@@ -833,7 +966,7 @@ private:
      * @return true if flag set or false in flag not set
      */
     bool getFlag(unsigned int flag_) const {
-        return bool((_flags & flag_) != 0);
+        return ((_flags & flag_) != 0);
     }
 
     /**
@@ -852,6 +985,9 @@ private:
     /** Update internal property cache about isStandardType() */
     void update_property_isStandardType();
 
+    /** Bitfield bit count. */
+    unsigned char _bits;
+
     // AST..
     Token *_astOperand1;
     Token *_astOperand2;
@@ -861,7 +997,11 @@ private:
     std::string* _originalName;
 
     // ValueType
-    ValueType *valuetype;
+    ValueType *_valuetype;
+
+    // ValueFlow
+    std::list<ValueFlow::Value>* _values;
+    static const std::list<ValueFlow::Value> emptyValueList;
 
 public:
     void astOperand1(Token *tok);
@@ -893,7 +1033,12 @@ public:
     bool isCalculation() const;
 
     void clearAst() {
-        _astOperand1 = _astOperand2 = _astParent = NULL;
+        _astOperand1 = _astOperand2 = _astParent = nullptr;
+    }
+
+    void clearValueFlow() {
+        delete _values;
+        _values = nullptr;
     }
 
     std::string astString(const char *sep = "") const {
