@@ -29,6 +29,7 @@
 #include <list>
 #include <map>
 #include <string>
+#include <stack>
 
 class Settings;
 class SymbolDatabase;
@@ -49,6 +50,34 @@ class CPPCHECKLIB Tokenizer {
     friend class TestSimplifyTypedef;
     friend class TestTokenizer;
     friend class SymbolDatabase;
+
+    /** Class used in Tokenizer::setVarIdPass1 */
+    class VariableMap {
+    private:
+        std::map<std::string, unsigned int> variableId;
+        std::stack<std::list<std::pair<std::string, unsigned int> > > scopeInfo;
+        mutable unsigned int varId;
+    public:
+        VariableMap();
+        void enterScope();
+        bool leaveScope();
+        void addVariable(const std::string &varname);
+        bool hasVariable(const std::string &varname) const;
+        std::map<std::string, unsigned int>::const_iterator find(const std::string &varname) const {
+            return variableId.find(varname);
+        }
+        std::map<std::string, unsigned int>::const_iterator end() const {
+            return variableId.end();
+        }
+        const std::map<std::string, unsigned int> &map() const {
+            return variableId;
+        }
+        unsigned int *getVarId() const {
+            return &varId;
+        }
+    };
+
+
 public:
     Tokenizer();
     Tokenizer(const Settings * settings, ErrorLogger *errorLogger);
@@ -681,6 +710,16 @@ private:
      * */
     bool simplifyStrlen();
 
+    /**
+     * Convert namespace aliases
+     */
+    void simplifyNamespaceAliases();
+
+    /**
+     * Convert C++17 style nested namespace to older style
+     */
+    void simplifyNestedNamespace();
+
     /**
     * Prepare ternary operators with parentheses so that the AST can be created
     * */
@@ -702,7 +741,7 @@ private:
     void unsupportedTypedef(const Token *tok) const;
 
     void setVarIdClassDeclaration(const Token * const startToken,
-                                  const std::map<std::string, unsigned int> &variableId,
+                                  const VariableMap &variableMap,
                                   const unsigned int scopeStartVarId,
                                   std::map<unsigned int, std::map<std::string,unsigned int> >& structMembers);
 
@@ -807,11 +846,11 @@ public:
     }
 
 private:
-    /** Disable copy constructor, no implementation */
-    Tokenizer(const Tokenizer &);
+    /** Disable copy constructor */
+    Tokenizer(const Tokenizer &) = delete;
 
-    /** Disable assignment operator, no implementation */
-    Tokenizer &operator=(const Tokenizer &);
+    /** Disable assignment operator */
+    Tokenizer &operator=(const Tokenizer &) = delete;
 
     Token *processFunc(Token *tok2, bool inOperator) const;
 
@@ -845,7 +884,7 @@ private:
     /** variable count */
     unsigned int _varId;
 
-    /** unnamed count "Unnamed0", "Unnamed1", "Unnamed2", .. */
+    /** unnamed count "Unnamed0", "Unnamed1", "Unnamed2", ... */
     unsigned int _unnamedCount;
 
     /**
